'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.help = exports.getScriptsAndArgs = undefined;

var _lodash = require('lodash.remove');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.contains');

var _lodash4 = _interopRequireDefault(_lodash3);

var _lodash5 = require('lodash.isplainobject');

var _lodash6 = _interopRequireDefault(_lodash5);

var _shellEscape = require('shell-escape');

var _shellEscape2 = _interopRequireDefault(_shellEscape);

var _lodash7 = require('lodash.isempty');

var _lodash8 = _interopRequireDefault(_lodash7);

var _safe = require('colors/safe');

var _safe2 = _interopRequireDefault(_safe);

var _resolveScriptObjectToString = require('./resolve-script-object-to-string');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

exports.getScriptsAndArgs = getScriptsAndArgs;
exports.help = help;


function getScriptsAndArgs(program) {
  var scripts = void 0,
      args = void 0,
      parallel = void 0;
  if (!(0, _lodash8.default)(program.parallel)) {
    scripts = program.parallel.split(',');
    args = getArgs(program.args, program.rawArgs, scripts);
    parallel = true;
  } else {
    scripts = program.args[0].split(',');
    args = getArgs(program.args.slice(1), program.rawArgs, scripts);
    parallel = false;
  }
  return { scripts: scripts, args: args, parallel: parallel };
}

function getArgs(args, rawArgs, scripts) {
  var allArgs = rawArgs.slice(2);
  var psArgs = ['-p', '--parallel', '-c', '--config'];
  var psFlags = ['-s', '--silent'];
  var cleanedArgs = (0, _lodash2.default)(allArgs, function (item, index, arry) {
    var isArgOrFlag = (0, _lodash4.default)(psArgs, item) || (0, _lodash4.default)(psFlags, item);
    var isArgValue = (0, _lodash4.default)(psArgs, arry[index - 1]);
    var isInScripts = (0, _lodash4.default)(scripts, item);
    return !isArgOrFlag && !isArgValue && !isInScripts;
  });
  return (0, _shellEscape2.default)(cleanedArgs);
}

function help(_ref) {
  var scripts = _ref.scripts;

  var availableScripts = getAvailableScripts(scripts);
  var scriptLines = availableScripts.map(function (_ref2) {
    var name = _ref2.name;
    var description = _ref2.description;
    var script = _ref2.script;

    var coloredName = _safe2.default.green(name);
    var coloredScript = _safe2.default.gray(script);
    var line = void 0;
    if (description) {
      line = [coloredName, _safe2.default.white(description), coloredScript];
    } else {
      line = [coloredName, coloredScript];
    }
    return line.join(' - ').trim();
  });
  if (!scriptLines.length) {
    return _safe2.default.yellow('There are no scripts available');
  } else {
    var topMessage = 'Available scripts (camel or kebab case accepted)';
    var message = topMessage + '\n\n' + scriptLines.join('\n');
    return message;
  }
}

function getAvailableScripts(config, prefix) {
  var excluded = ['description', 'script', 'default'];
  return Object.keys(config).reduce(function (scripts, key) {
    var val = config[key];
    if ((0, _lodash4.default)(excluded, key)) {
      return scripts;
    }
    var scriptObj = (0, _resolveScriptObjectToString.resolveScriptObjectToScript)(val);
    if (scriptObj) {
      var description = scriptObj.description;
      var script = scriptObj.script;

      var prefixed = prefix ? [prefix, key] : [key];
      scripts = [].concat(_toConsumableArray(scripts), [{ name: prefixed.join('.'), description: description, script: script }]);
    }
    if ((0, _lodash6.default)(val)) {
      return [].concat(_toConsumableArray(scripts), _toConsumableArray(getAvailableScripts(val, key)));
    }
    return scripts;
  }, []);
}